@@ -20,12 +20,14 @@ use crate::Locator;
 use crate::fs::relativize_path;
 use crate::registry::Rule;
 use crate::rule_redirects::get_redirect_target;
+use crate::suppression::Suppressions;
 
 /// Generates an array of edits that matches the length of `messages`.
 /// Each potential edit in the array is paired, in order, with the associated diagnostic.
 /// Each edit will add a `noqa` comment to the appropriate line in the source to hide
 /// the diagnostic. These edits may conflict with each other and should not be applied
 /// simultaneously.
+#[expect(clippy::too_many_arguments)]
 pub fn generate_noqa_edits(
     path: &Path,
     diagnostics: &[Diagnostic],
@@ -34,11 +36,19 @@ pub fn generate_noqa_edits(
     external: &[String],
     noqa_line_for: &NoqaMapping,
     line_ending: LineEnding,
+    suppressions: &Suppressions,
 ) -> Vec<Option<Edit>> {
     let file_directives = FileNoqaDirectives::extract(locator, comment_ranges, external, path);
     let exemption = FileExemption::from(&file_directives);
     let directives = NoqaDirectives::from_commented_ranges(comment_ranges, external, path, locator);
-    let comments = find_noqa_comments(diagnostics, locator, &exemption, &directives, noqa_line_for);
+    let comments = find_noqa_comments(
+        diagnostics,
+        locator,
+        &exemption,
+        &directives,
+        noqa_line_for,
+        suppressions,
+    );
     build_noqa_edits_by_diagnostic(comments, locator, line_ending, None)
 }
 
@@ -725,6 +735,7 @@ pub(crate) fn add_noqa(
     noqa_line_for: &NoqaMapping,
     line_ending: LineEnding,
     reason: Option<&str>,
+    suppressions: &Suppressions,
 ) -> Result<usize> {
     let (count, output) = add_noqa_inner(
         path,
@@ -735,6 +746,7 @@ pub(crate) fn add_noqa(
         noqa_line_for,
         line_ending,
         reason,
+        suppressions,
     );
 
     fs::write(path, output)?;
@@ -751,6 +763,7 @@ fn add_noqa_inner(
     noqa_line_for: &NoqaMapping,
     line_ending: LineEnding,
     reason: Option<&str>,
+    suppressions: &Suppressions,
 ) -> (usize, String) {
     let mut count = 0;
 
@@ -760,7 +773,14 @@ fn add_noqa_inner(
 
     let directives = NoqaDirectives::from_commented_ranges(comment_ranges, external, path, locator);
 
-    let comments = find_noqa_comments(diagnostics, locator, &exemption, &directives, noqa_line_for);
+    let comments = find_noqa_comments(
+        diagnostics,
+        locator,
+        &exemption,
+        &directives,
+        noqa_line_for,
+        suppressions,
+    );
 
     let edits = build_noqa_edits_by_line(comments, locator, line_ending, reason);
 
@@ -859,6 +879,7 @@ fn find_noqa_comments<'a>(
     exemption: &'a FileExemption,
     directives: &'a NoqaDirectives,
     noqa_line_for: &NoqaMapping,
+    suppressions: &'a Suppressions,
 ) -> Vec<Option<NoqaComment<'a>>> {
     // List of noqa comments, ordered to match up with `messages`
     let mut comments_by_line: Vec<Option<NoqaComment<'a>>> = vec![];
@@ -875,6 +896,12 @@ fn find_noqa_comments<'a>(
             continue;
         }
 
+        // Apply ranged suppressions next
+        if suppressions.check_diagnostic(message) {
+            comments_by_line.push(None);
+            continue;
+        }
+
         // Is the violation ignored by a `noqa` directive on the parent line?
         if let Some(parent) = message.parent() {
             if let Some(directive_line) =
@@ -1253,6 +1280,7 @@ mod tests {
     use crate::rules::pycodestyle::rules::{AmbiguousVariableName, UselessSemicolon};
     use crate::rules::pyflakes::rules::UnusedVariable;
     use crate::rules::pyupgrade::rules::PrintfStringFormatting;
+    use crate::suppression::Suppressions;
     use crate::{Edit, Violation};
     use crate::{Locator, generate_noqa_edits};
 
@@ -1298,7 +1326,7 @@ mod tests {
     fn noqa_all() {
         let source = "# noqa";
         let directive = lex_inline_noqa(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(directive, @r"
+        assert_debug_snapshot!(directive, @"
         Ok(
             Some(
                 NoqaLexerOutput {
@@ -1319,7 +1347,7 @@ mod tests {
     fn noqa_no_code() {
         let source = "# noqa:";
         let directive = lex_inline_noqa(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(directive, @r"
+        assert_debug_snapshot!(directive, @"
         Err(
             MissingCodes,
         )
@@ -1331,7 +1359,7 @@ mod tests {
     fn noqa_no_code_invalid_suffix() {
         let source = "# noqa: foo";
         let directive = lex_inline_noqa(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(directive, @r"
+        assert_debug_snapshot!(directive, @"
         Err(
             MissingCodes,
         )
@@ -1343,7 +1371,7 @@ mod tests {
     fn noqa_no_code_trailing_content() {
         let source = "# noqa:  # Foo";
         let directive = lex_inline_noqa(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(directive, @r"
+        assert_debug_snapshot!(directive, @"
         Err(
             MissingCodes,
         )
@@ -1355,7 +1383,7 @@ mod tests {
     fn malformed_code_1() {
         let source = "# noqa: F";
         let directive = lex_inline_noqa(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(directive, @r"
+        assert_debug_snapshot!(directive, @"
         Err(
             MissingCodes,
         )
@@ -1394,7 +1422,7 @@ mod tests {
     fn malformed_code_3() {
         let source = "# noqa: RUF001F";
         let directive = lex_inline_noqa(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(directive, @r"
+        assert_debug_snapshot!(directive, @"
         Err(
             InvalidCodeSuffix,
         )
@@ -1464,7 +1492,7 @@ mod tests {
     fn noqa_all_case_insensitive() {
         let source = "# NOQA";
         let directive = lex_inline_noqa(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(directive, @r"
+        assert_debug_snapshot!(directive, @"
         Ok(
             Some(
                 NoqaLexerOutput {
@@ -1597,7 +1625,7 @@ mod tests {
     fn noqa_all_no_space() {
         let source = "#noqa";
         let directive = lex_inline_noqa(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(directive, @r"
+        assert_debug_snapshot!(directive, @"
         Ok(
             Some(
                 NoqaLexerOutput {
@@ -1676,7 +1704,7 @@ mod tests {
     fn noqa_all_multi_space() {
         let source = "#  noqa";
         let directive = lex_inline_noqa(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(directive, @r"
+        assert_debug_snapshot!(directive, @"
         Ok(
             Some(
                 NoqaLexerOutput {
@@ -1809,7 +1837,7 @@ mod tests {
     fn noqa_all_leading_comment() {
         let source = "# Some comment describing the noqa # noqa";
         let directive = lex_inline_noqa(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(directive, @r"
+        assert_debug_snapshot!(directive, @"
         Ok(
             Some(
                 NoqaLexerOutput {
@@ -1888,7 +1916,7 @@ mod tests {
     fn noqa_all_trailing_comment() {
         let source = "# noqa # Some comment describing the noqa";
         let directive = lex_inline_noqa(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(directive, @r"
+        assert_debug_snapshot!(directive, @"
         Ok(
             Some(
                 NoqaLexerOutput {
@@ -1967,7 +1995,7 @@ mod tests {
     fn noqa_invalid_codes() {
         let source = "# noqa: unused-import, F401, some other code";
         let directive = lex_inline_noqa(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(directive, @r"
+        assert_debug_snapshot!(directive, @"
         Err(
             MissingCodes,
         )
@@ -2111,7 +2139,7 @@ mod tests {
     fn noqa_code_invalid_code_suffix() {
         let source = "# noqa: F401abc";
         let directive = lex_inline_noqa(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(directive, @r"
+        assert_debug_snapshot!(directive, @"
         Err(
             InvalidCodeSuffix,
         )
@@ -2123,7 +2151,7 @@ mod tests {
     fn noqa_invalid_suffix() {
         let source = "# noqa[F401]";
         let directive = lex_inline_noqa(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(directive, @r"
+        assert_debug_snapshot!(directive, @"
         Err(
             InvalidSuffix,
         )
@@ -2135,7 +2163,7 @@ mod tests {
     fn flake8_exemption_all() {
         let source = "# flake8: noqa";
         let exemption = lex_file_exemption(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(exemption, @r"
+        assert_debug_snapshot!(exemption, @"
         Ok(
             Some(
                 NoqaLexerOutput {
@@ -2156,7 +2184,7 @@ mod tests {
     fn flake8_noqa_no_code() {
         let source = "# flake8: noqa:";
         let exemption = lex_file_exemption(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(exemption, @r"
+        assert_debug_snapshot!(exemption, @"
         Err(
             MissingCodes,
         )
@@ -2168,7 +2196,7 @@ mod tests {
     fn flake8_noqa_no_code_invalid_suffix() {
         let source = "# flake8: noqa: foo";
         let exemption = lex_file_exemption(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(exemption, @r"
+        assert_debug_snapshot!(exemption, @"
         Err(
             MissingCodes,
         )
@@ -2180,7 +2208,7 @@ mod tests {
     fn flake8_noqa_no_code_trailing_content() {
         let source = "# flake8: noqa:  # Foo";
         let exemption = lex_file_exemption(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(exemption, @r"
+        assert_debug_snapshot!(exemption, @"
         Err(
             MissingCodes,
         )
@@ -2192,7 +2220,7 @@ mod tests {
     fn flake8_malformed_code_1() {
         let source = "# flake8: noqa: F";
         let directive = lex_file_exemption(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(directive, @r"
+        assert_debug_snapshot!(directive, @"
         Err(
             MissingCodes,
         )
@@ -2231,7 +2259,7 @@ mod tests {
     fn flake8_malformed_code_3() {
         let source = "# flake8: noqa: RUF001F";
         let directive = lex_file_exemption(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(directive, @r"
+        assert_debug_snapshot!(directive, @"
         Err(
             InvalidCodeSuffix,
         )
@@ -2243,7 +2271,7 @@ mod tests {
     fn ruff_exemption_all() {
         let source = "# ruff: noqa";
         let exemption = lex_file_exemption(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(exemption, @r"
+        assert_debug_snapshot!(exemption, @"
         Ok(
             Some(
                 NoqaLexerOutput {
@@ -2264,7 +2292,7 @@ mod tests {
     fn ruff_noqa_no_code() {
         let source = "# ruff: noqa:";
         let exemption = lex_file_exemption(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(exemption, @r"
+        assert_debug_snapshot!(exemption, @"
         Err(
             MissingCodes,
         )
@@ -2276,7 +2304,7 @@ mod tests {
     fn ruff_noqa_no_code_invalid_suffix() {
         let source = "# ruff: noqa: foo";
         let exemption = lex_file_exemption(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(exemption, @r"
+        assert_debug_snapshot!(exemption, @"
         Err(
             MissingCodes,
         )
@@ -2288,7 +2316,7 @@ mod tests {
     fn ruff_noqa_no_code_trailing_content() {
         let source = "# ruff: noqa:  # Foo";
         let exemption = lex_file_exemption(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(exemption, @r"
+        assert_debug_snapshot!(exemption, @"
         Err(
             MissingCodes,
         )
@@ -2300,7 +2328,7 @@ mod tests {
     fn ruff_malformed_code_1() {
         let source = "# ruff: noqa: F";
         let directive = lex_file_exemption(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(directive, @r"
+        assert_debug_snapshot!(directive, @"
         Err(
             MissingCodes,
         )
@@ -2339,7 +2367,7 @@ mod tests {
     fn ruff_malformed_code_3() {
         let source = "# ruff: noqa: RUF001F";
         let directive = lex_file_exemption(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(directive, @r"
+        assert_debug_snapshot!(directive, @"
         Err(
             InvalidCodeSuffix,
         )
@@ -2351,7 +2379,7 @@ mod tests {
     fn flake8_exemption_all_no_space() {
         let source = "#flake8:noqa";
         let exemption = lex_file_exemption(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(exemption, @r"
+        assert_debug_snapshot!(exemption, @"
         Ok(
             Some(
                 NoqaLexerOutput {
@@ -2372,7 +2400,7 @@ mod tests {
     fn ruff_exemption_all_no_space() {
         let source = "#ruff:noqa";
         let exemption = lex_file_exemption(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(exemption, @r"
+        assert_debug_snapshot!(exemption, @"
         Ok(
             Some(
                 NoqaLexerOutput {
@@ -2591,7 +2619,7 @@ mod tests {
     fn ruff_exemption_invalid_code_suffix() {
         let source = "# ruff: noqa: F401abc";
         let exemption = lex_file_exemption(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(exemption, @r"
+        assert_debug_snapshot!(exemption, @"
         Err(
             InvalidCodeSuffix,
         )
@@ -2657,7 +2685,7 @@ mod tests {
     fn ruff_exemption_all_leading_comment() {
         let source = "# Leading comment # ruff: noqa";
         let exemption = lex_file_exemption(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(exemption, @r"
+        assert_debug_snapshot!(exemption, @"
         Ok(
             Some(
                 NoqaLexerOutput {
@@ -2678,7 +2706,7 @@ mod tests {
     fn ruff_exemption_all_trailing_comment() {
         let source = "# ruff: noqa # Trailing comment";
         let exemption = lex_file_exemption(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(exemption, @r"
+        assert_debug_snapshot!(exemption, @"
         Ok(
             Some(
                 NoqaLexerOutput {
@@ -2726,7 +2754,7 @@ mod tests {
     fn ruff_exemption_all_trailing_comment_no_space() {
         let source = "# ruff: noqa# Trailing comment";
         let exemption = lex_file_exemption(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(exemption, @r"
+        assert_debug_snapshot!(exemption, @"
         Ok(
             Some(
                 NoqaLexerOutput {
@@ -2747,7 +2775,7 @@ mod tests {
     fn ruff_exemption_all_trailing_comment_no_hash() {
         let source = "# ruff: noqa Trailing comment";
         let exemption = lex_file_exemption(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(exemption, @r"
+        assert_debug_snapshot!(exemption, @"
         Ok(
             Some(
                 NoqaLexerOutput {
@@ -2795,7 +2823,7 @@ mod tests {
     fn flake8_exemption_all_case_insensitive() {
         let source = "# flake8: NoQa";
         let exemption = lex_file_exemption(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(exemption, @r"
+        assert_debug_snapshot!(exemption, @"
         Ok(
             Some(
                 NoqaLexerOutput {
@@ -2816,7 +2844,7 @@ mod tests {
     fn ruff_exemption_all_case_insensitive() {
         let source = "# ruff: NoQa";
         let exemption = lex_file_exemption(TextRange::up_to(source.text_len()), source);
-        assert_debug_snapshot!(exemption, @r"
+        assert_debug_snapshot!(exemption, @"
         Ok(
             Some(
                 NoqaLexerOutput {
@@ -2848,6 +2876,7 @@ mod tests {
             &noqa_line_for,
             LineEnding::Lf,
             None,
+            &Suppressions::default(),
         );
         assert_eq!(count, 0);
         assert_eq!(output, format!("{contents}"));
@@ -2872,6 +2901,7 @@ mod tests {
             &noqa_line_for,
             LineEnding::Lf,
             None,
+            &Suppressions::default(),
         );
         assert_eq!(count, 1);
         assert_eq!(output, "x = 1  # noqa: F841\n");
@@ -2903,6 +2933,7 @@ mod tests {
             &noqa_line_for,
             LineEnding::Lf,
             None,
+            &Suppressions::default(),
         );
         assert_eq!(count, 1);
         assert_eq!(output, "x = 1  # noqa: E741, F841\n");
@@ -2934,6 +2965,7 @@ mod tests {
             &noqa_line_for,
             LineEnding::Lf,
             None,
+            &Suppressions::default(),
         );
         assert_eq!(count, 0);
         assert_eq!(output, "x = 1  # noqa");
@@ -2956,6 +2988,7 @@ print(
         let messages = [PrintfStringFormatting
             .into_diagnostic(TextRange::new(12.into(), 79.into()), &source_file)];
         let comment_ranges = CommentRanges::default();
+        let suppressions = Suppressions::default();
         let edits = generate_noqa_edits(
             path,
             &messages,
@@ -2964,6 +2997,7 @@ print(
             &[],
             &noqa_line_for,
             LineEnding::Lf,
+            &suppressions,
         );
         assert_eq!(
             edits,
@@ -2987,6 +3021,7 @@ bar =
             [UselessSemicolon.into_diagnostic(TextRange::new(4.into(), 5.into()), &source_file)];
         let noqa_line_for = NoqaMapping::default();
         let comment_ranges = CommentRanges::default();
+        let suppressions = Suppressions::default();
         let edits = generate_noqa_edits(
             path,
             &messages,
@@ -2995,6 +3030,7 @@ bar =
             &[],
             &noqa_line_for,
             LineEnding::Lf,
+            &suppressions,
         );
         assert_eq!(
             edits,
