@@ -1,15 +1,16 @@
+use crate::Db;
 use crate::db::tests::TestDb;
 use crate::place::{builtins_symbol, known_module_symbol};
 use crate::types::enums::is_single_member_enum;
 use crate::types::tuple::TupleType;
 use crate::types::{
-    BoundMethodType, CallableType, EnumLiteralType, IntersectionBuilder, KnownClass, Parameter,
+    BoundMethodType, EnumLiteralType, IntersectionBuilder, IntersectionType, KnownClass, Parameter,
     Parameters, Signature, SpecialFormType, SubclassOfType, Type, UnionType,
 };
-use crate::{Db, module_resolver::KnownModule};
-use hashbrown::HashSet;
 use quickcheck::{Arbitrary, Gen};
 use ruff_python_ast::name::Name;
+use rustc_hash::FxHashSet;
+use ty_module_resolver::KnownModule;
 
 /// A test representation of a type that can be transformed unambiguously into a real Type,
 /// given a db.
@@ -58,7 +59,7 @@ pub(crate) enum Ty {
     },
     Callable {
         params: CallableParams,
-        returns: Option<Box<Ty>>,
+        returns: Box<Ty>,
     },
     /// `unittest.mock.Mock` is interesting because it is a nominal instance type
     /// where the class has `Any` in its MRO
@@ -76,24 +77,27 @@ impl CallableParams {
     pub(crate) fn into_parameters(self, db: &TestDb) -> Parameters<'_> {
         match self {
             CallableParams::GradualForm => Parameters::gradual_form(),
-            CallableParams::List(params) => Parameters::new(params.into_iter().map(|param| {
-                let mut parameter = match param.kind {
-                    ParamKind::PositionalOnly => Parameter::positional_only(param.name),
-                    ParamKind::PositionalOrKeyword => {
-                        Parameter::positional_or_keyword(param.name.unwrap())
+            CallableParams::List(params) => Parameters::new(
+                db,
+                params.into_iter().map(|param| {
+                    let mut parameter = match param.kind {
+                        ParamKind::PositionalOnly => Parameter::positional_only(param.name),
+                        ParamKind::PositionalOrKeyword => {
+                            Parameter::positional_or_keyword(param.name.unwrap())
+                        }
+                        ParamKind::Variadic => Parameter::variadic(param.name.unwrap()),
+                        ParamKind::KeywordOnly => Parameter::keyword_only(param.name.unwrap()),
+                        ParamKind::KeywordVariadic => {
+                            Parameter::keyword_variadic(param.name.unwrap())
+                        }
+                    };
+                    parameter = parameter.with_annotated_type(param.annotated_ty.into_type(db));
+                    if let Some(default_ty) = param.default_ty {
+                        parameter = parameter.with_default_type(default_ty.into_type(db));
                     }
-                    ParamKind::Variadic => Parameter::variadic(param.name.unwrap()),
-                    ParamKind::KeywordOnly => Parameter::keyword_only(param.name.unwrap()),
-                    ParamKind::KeywordVariadic => Parameter::keyword_variadic(param.name.unwrap()),
-                };
-                if let Some(annotated_ty) = param.annotated_ty {
-                    parameter = parameter.with_annotated_type(annotated_ty.into_type(db));
-                }
-                if let Some(default_ty) = param.default_ty {
-                    parameter = parameter.with_default_type(default_ty.into_type(db));
-                }
-                parameter
-            })),
+                    parameter
+                }),
+            ),
         }
     }
 }
@@ -102,7 +106,7 @@ impl CallableParams {
 pub(crate) struct Param {
     kind: ParamKind,
     name: Option<Name>,
-    annotated_ty: Option<Ty>,
+    annotated_ty: Ty,
     default_ty: Option<Ty>,
 }
 
@@ -229,12 +233,9 @@ impl Ty {
 
                 create_bound_method(db, function, builtins_class)
             }
-            Ty::Callable { params, returns } => CallableType::single(
+            Ty::Callable { params, returns } => Type::single_callable(
                 db,
-                Signature::new(
-                    params.into_parameters(db),
-                    returns.map(|ty| ty.into_type(db)),
-                ),
+                Signature::new(params.into_parameters(db), returns.into_type(db)),
             ),
         }
     }
@@ -368,7 +369,7 @@ fn arbitrary_type(g: &mut Gen, size: u32, fully_static: bool) -> Ty {
                     0 if !fully_static => CallableParams::GradualForm,
                     _ => CallableParams::List(arbitrary_parameter_list(g, size, fully_static)),
                 },
-                returns: arbitrary_annotation(g, size - 1, fully_static).map(Box::new),
+                returns: Box::new(arbitrary_type(g, size - 1, fully_static)),
             },
             _ => unreachable!(),
         }
@@ -377,7 +378,7 @@ fn arbitrary_type(g: &mut Gen, size: u32, fully_static: bool) -> Ty {
 
 fn arbitrary_parameter_list(g: &mut Gen, size: u32, fully_static: bool) -> Vec<Param> {
     let mut params: Vec<Param> = vec![];
-    let mut used_names = HashSet::new();
+    let mut used_names = FxHashSet::default();
 
     // First, choose the number of parameters to generate.
     for _ in 0..*g.choose(&[0, 1, 2, 3, 4, 5]).unwrap() {
@@ -427,7 +428,7 @@ fn arbitrary_parameter_list(g: &mut Gen, size: u32, fully_static: bool) -> Vec<P
         params.push(Param {
             kind: next_kind,
             name,
-            annotated_ty: arbitrary_annotation(g, size, fully_static),
+            annotated_ty: arbitrary_type(g, size, fully_static),
             default_ty: if matches!(next_kind, ParamKind::Variadic | ParamKind::KeywordVariadic) {
                 None
             } else {
@@ -439,15 +440,6 @@ fn arbitrary_parameter_list(g: &mut Gen, size: u32, fully_static: bool) -> Vec<P
     params
 }
 
-/// An arbitrary optional type, always `Some` if fully static.
-fn arbitrary_annotation(g: &mut Gen, size: u32, fully_static: bool) -> Option<Ty> {
-    if fully_static {
-        Some(arbitrary_type(g, size, true))
-    } else {
-        arbitrary_optional_type(g, size, false)
-    }
-}
-
 fn arbitrary_optional_type(g: &mut Gen, size: u32, fully_static: bool) -> Option<Ty> {
     match u32::arbitrary(g) % 2 {
         0 => None,
@@ -573,11 +565,7 @@ pub(crate) fn intersection<'db>(
     db: &'db TestDb,
     tys: impl IntoIterator<Item = Type<'db>>,
 ) -> Type<'db> {
-    let mut builder = IntersectionBuilder::new(db);
-    for ty in tys {
-        builder = builder.add_positive(ty);
-    }
-    builder.build()
+    IntersectionType::from_elements(db, tys)
 }
 
 pub(crate) fn union<'db>(db: &'db TestDb, tys: impl IntoIterator<Item = Type<'db>>) -> Type<'db> {
