@@ -1,15 +1,21 @@
 //! Match [`Diagnostic`]s against assertions and produce test failure
 //! messages for any mismatches.
-use crate::assertion::{InlineFileAssertions, ParsedAssertion, UnparsedAssertion};
-use crate::db::Db;
-use crate::diagnostic::SortedDiagnostics;
+
+use std::borrow::Cow;
+use std::cmp::Ordering;
+use std::ops::Range;
+use std::sync::LazyLock;
+
 use colored::Colorize;
+use path_slash::PathExt;
 use ruff_db::diagnostic::{Diagnostic, DiagnosticId};
 use ruff_db::files::File;
 use ruff_db::source::{SourceText, line_index, source_text};
 use ruff_source_file::{LineIndex, OneIndexed};
-use std::cmp::Ordering;
-use std::ops::Range;
+
+use crate::assertion::{InlineFileAssertions, ParsedAssertion, UnparsedAssertion};
+use crate::db::Db;
+use crate::diagnostic::SortedDiagnostics;
 
 #[derive(Debug, Default)]
 pub(super) struct FailuresByLine {
@@ -194,12 +200,55 @@ impl UnmatchedWithColumn for &Diagnostic {
 
 /// Discard `@Todo`-type metadata from expected types, which is not available
 /// when running in release mode.
-#[cfg(not(debug_assertions))]
-fn discard_todo_metadata(ty: &str) -> std::borrow::Cow<'_, str> {
-    static TODO_METADATA_REGEX: std::sync::LazyLock<regex::Regex> =
-        std::sync::LazyLock::new(|| regex::Regex::new(r"@Todo\([^)]*\)").unwrap());
+///
+/// Some `@Todo` variants (like `@Todo(StarredExpression)` and `@Todo(typing.Unpack)`)
+/// are hardcoded enum variants that always display their message, so we preserve those.
+fn discard_todo_metadata(ty: &str) -> Cow<'_, str> {
+    #[cfg(not(debug_assertions))]
+    {
+        /// `@Todo` variants that are hardcoded and always display their message,
+        /// even in release mode.
+        const PRESERVED_TODO_VARIANTS: &[&str] =
+            &["@Todo(StarredExpression)", "@Todo(typing.Unpack)"];
 
-    TODO_METADATA_REGEX.replace_all(ty, "@Todo")
+        static TODO_METADATA_REGEX: LazyLock<regex::Regex> =
+            LazyLock::new(|| regex::Regex::new(r"@Todo\([^)]*\)").unwrap());
+
+        TODO_METADATA_REGEX.replace_all(ty, |caps: &regex::Captures| {
+            let matched = caps.get(0).unwrap().as_str();
+            if PRESERVED_TODO_VARIANTS.contains(&matched) {
+                matched.to_string()
+            } else {
+                "@Todo".to_string()
+            }
+        })
+    }
+
+    #[cfg(debug_assertions)]
+    Cow::Borrowed(ty)
+}
+
+/// Normalize paths in diagnostics to Unix paths before comparing them against
+/// the expected type. Doing otherwise means that it's hard to write cross-platform
+/// tests, since in some edge cases the display of a type can include a path to the
+/// file in which the type was defined (e.g. `foo.bar.A @ src/foo/bar.py:10:5` on Unix,
+/// but `foo.bar.A @ src\foo\bar.py:10:5` on Windows).
+fn normalize_paths(ty: &str) -> Cow<'_, str> {
+    static PATH_IN_CLASS_DISPLAY_REGEX: LazyLock<regex::Regex> =
+        LazyLock::new(|| regex::Regex::new(r"( @ )([^\.]+?)(\.pyi?:\d)").unwrap());
+
+    fn normalize_path_captures(path_captures: &regex::Captures) -> String {
+        let normalized_path = std::path::Path::new(&path_captures[2])
+            .to_slash()
+            .expect("Python module paths should be valid UTF-8");
+
+        format!(
+            "{}{}{}",
+            &path_captures[1], normalized_path, &path_captures[3]
+        )
+    }
+
+    PATH_IN_CLASS_DISPLAY_REGEX.replace_all(ty, normalize_path_captures)
 }
 
 struct Matcher {
@@ -285,7 +334,7 @@ impl Matcher {
                         .column
                         .is_none_or(|col| col == self.column(diagnostic));
                     let message_matches = error.message_contains.is_none_or(|needle| {
-                        diagnostic.concise_message().to_string().contains(needle)
+                        normalize_paths(&diagnostic.concise_message().to_str()).contains(needle)
                     });
                     lint_name_matches && column_matches && message_matches
                 });
@@ -297,21 +346,47 @@ impl Matcher {
                 }
             }
             ParsedAssertion::Revealed(expected_type) => {
-                #[cfg(not(debug_assertions))]
-                let expected_type = discard_todo_metadata(&expected_type);
+                let expected_type = discard_todo_metadata(expected_type);
+                let expected_reveal_type_message = format!("`{expected_type}`");
+
+                let diagnostic_matches_reveal = |diagnostic: &Diagnostic| {
+                    if diagnostic.id() != DiagnosticId::RevealedType {
+                        return false;
+                    }
+                    let primary_message = diagnostic.primary_message();
+                    let Some(primary_annotation) =
+                        (diagnostic.primary_annotation()).and_then(|a| a.get_message())
+                    else {
+                        return false;
+                    };
+
+                    let primary_annotation = normalize_paths(primary_annotation);
+
+                    // reveal_type, reveal_protocol_interface
+                    if matches!(
+                        primary_message,
+                        "Revealed type" | "Revealed protocol interface"
+                    ) && primary_annotation == expected_reveal_type_message
+                    {
+                        return true;
+                    }
+
+                    // reveal_when_assignable_to, reveal_when_subtype_of, reveal_mro
+                    if matches!(
+                        primary_message,
+                        "Assignability holds" | "Subtyping holds" | "Revealed MRO"
+                    ) && primary_annotation == expected_type
+                    {
+                        return true;
+                    }
+
+                    false
+                };
 
                 let mut matched_revealed_type = None;
                 let mut matched_undefined_reveal = None;
-                let expected_reveal_type_message = format!("`{expected_type}`");
                 for (index, diagnostic) in unmatched.iter().enumerate() {
-                    if matched_revealed_type.is_none()
-                        && diagnostic.id() == DiagnosticId::RevealedType
-                        && diagnostic
-                            .primary_annotation()
-                            .and_then(|a| a.get_message())
-                            .unwrap_or_default()
-                            == expected_reveal_type_message
-                    {
+                    if matched_revealed_type.is_none() && diagnostic_matches_reveal(diagnostic) {
                         matched_revealed_type = Some(index);
                     } else if matched_undefined_reveal.is_none()
                         && diagnostic.id().is_lint_named("undefined-reveal")
@@ -345,9 +420,8 @@ mod tests {
     use ruff_python_trivia::textwrap::dedent;
     use ruff_source_file::OneIndexed;
     use ruff_text_size::TextRange;
-    use ty_python_semantic::{
-        Program, ProgramSettings, PythonPlatform, PythonVersionWithSource, SearchPathSettings,
-    };
+    use ty_module_resolver::SearchPathSettings;
+    use ty_python_semantic::{Program, ProgramSettings, PythonPlatform, PythonVersionWithSource};
 
     struct ExpectedDiagnostic {
         id: DiagnosticId,
@@ -369,10 +443,16 @@ mod tests {
             let mut diag = if self.id == DiagnosticId::RevealedType {
                 Diagnostic::new(self.id, Severity::Error, "Revealed type")
             } else {
-                Diagnostic::new(self.id, Severity::Error, "")
+                Diagnostic::new(self.id, Severity::Error, self.message)
             };
             let span = Span::from(file).with_range(self.range);
-            diag.annotate(Annotation::primary(span).message(self.message));
+            let mut annotation = Annotation::primary(span);
+
+            if self.id == DiagnosticId::RevealedType {
+                annotation = annotation.message(self.message);
+            }
+
+            diag.annotate(annotation);
             diag
         }
     }
